Stan Math Library: stan/math/prim/scal/prob/binomial_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

binomial_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_BINOMIAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_BINOMIAL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_bounded.hpp>
9 #include <stan/math/prim/scal/err/check_finite.hpp>
10 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
11 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
12 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/fun/inv_logit.hpp>
15 #include <stan/math/prim/scal/fun/log1m.hpp>
16 #include <stan/math/prim/scal/fun/log_inv_logit.hpp>
17 #include <stan/math/prim/scal/fun/multiply_log.hpp>
18 #include <stan/math/prim/scal/fun/value_of.hpp>
19 #include <stan/math/prim/scal/fun/binomial_coefficient_log.hpp>
20 #include <stan/math/prim/scal/fun/lbeta.hpp>
21 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
22 #include <stan/math/prim/scal/meta/include_summand.hpp>
23 #include <stan/math/prim/scal/fun/inc_beta.hpp>
24 #include <boost/random/binomial_distribution.hpp>
25 #include <boost/random/variate_generator.hpp>
26
27
28 namespace stan {
29
30 namespace math {
31
32 // Binomial(n|N, theta) [N >= 0; 0 <= n <= N; 0 <= theta <= 1]
33 template <bool propto,
34 typename T_n,
35 typename T_N,
36 typename T_prob>
37 typename return_type<T_prob>::type
38 binomial_log(const T_n& n,
39 const T_N& N,
40 const T_prob& theta) {
41 typedef typename stan::partials_return_type<T_n, T_N, T_prob>::type
42 T_partials_return;
43
44 static const char* function("stan::math::binomial_log");
45
46 using stan::math::check_finite;
47 using stan::math::check_bounded;
48 using stan::math::check_nonnegative;
49 using stan::math::value_of;
50 using stan::math::check_consistent_sizes;
51 using stan::math::include_summand;
52
53 // check if any vectors are zero length
54 if (!(stan::length(n)
55 && stan::length(N)
56 && stan::length(theta)))
57 return 0.0;
58
59 T_partials_return logp = 0;
60 check_bounded(function, "Successes variable", n, 0, N);
61 check_nonnegative(function, "Population size parameter", N);
62 check_finite(function, "Probability parameter", theta);
63 check_bounded(function, "Probability parameter", theta, 0.0, 1.0);
64 check_consistent_sizes(function,
65 "Successes variable", n,
66 "Population size parameter", N,
67 "Probability parameter", theta);
68
69
70 // check if no variables are involved and prop-to
71 if (!include_summand<propto, T_prob>::value)
72 return 0.0;
73
74 // set up template expressions wrapping scalars into vector views
75 VectorView<const T_n> n_vec(n);
76 VectorView<const T_N> N_vec(N);
77 VectorView<const T_prob> theta_vec(theta);
78 size_t size = max_size(n, N, theta);
79
80 OperandsAndPartials<T_prob> operands_and_partials(theta);
81
82 using stan::math::multiply_log;
83 using stan::math::binomial_coefficient_log;
84 using stan::math::log1m;
85
86 if (include_summand<propto>::value) {
87 for (size_t i = 0; i < size; ++i)
88 logp += binomial_coefficient_log(N_vec[i], n_vec[i]);
89 }
90
91 VectorBuilder<true, T_partials_return, T_prob> log1m_theta(length(theta));
92 for (size_t i = 0; i < length(theta); ++i)
93 log1m_theta[i] = log1m(value_of(theta_vec[i]));
94
95 // no test for include_summand because return if not live
96 for (size_t i = 0; i < size; ++i)
97 logp += multiply_log(n_vec[i], value_of(theta_vec[i]))
98 + (N_vec[i] - n_vec[i]) * log1m_theta[i];
99
100 if (length(theta) == 1) {
101 T_partials_return temp1 = 0;
102 T_partials_return temp2 = 0;
103 for (size_t i = 0; i < size; ++i) {
104 temp1 += n_vec[i];
105 temp2 += N_vec[i] - n_vec[i];
106 }
107 if (!is_constant_struct<T_prob>::value) {
108 operands_and_partials.d_x1[0]
109 += temp1 / value_of(theta_vec[0])
110 - temp2 / (1.0 - value_of(theta_vec[0]));
111 }
112 } else {
113 if (!is_constant_struct<T_prob>::value) {
114 for (size_t i = 0; i < size; ++i)
115 operands_and_partials.d_x1[i]
116 += n_vec[i] / value_of(theta_vec[i])
117 - (N_vec[i] - n_vec[i]) / (1.0 - value_of(theta_vec[i]));
118 }
119 }
120
121 return operands_and_partials.value(logp);
122 }
123
124 template <typename T_n,
125 typename T_N,
126 typename T_prob>
127 inline
128 typename return_type<T_prob>::type
129 binomial_log(const T_n& n,
130 const T_N& N,
131 const T_prob& theta) {
132 return binomial_log<false>(n, N, theta);
133 }
134
135 }
136 }
137 #endif
stan::math::binomial_coefficient_log
fvar< T > binomial_coefficient_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: binomial_coefficient_log.hpp:16
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::check_bounded
bool check_bounded(const char *function, const char *name, const T_y &y, const T_low &low, const T_high &high)
Return true if the value is between the low and high values, inclusively.
Definition: check_bounded.hpp:95
stan::math::binomial_log
return_type< T_prob >::type binomial_log(const T_n &n, const T_N &N, const T_prob &theta)
Definition: binomial_log.hpp:38
binomial_coefficient_log.hpp
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
log_inv_logit.hpp
check_less_or_equal.hpp
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
check_bounded.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
lbeta.hpp
inc_beta.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
inv_logit.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::log1m
fvar< T > log1m(const fvar< T > &x)
Definition: log1m.hpp:16
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.